@@ -30,7 +30,6 @@ typedef struct _GtkHTMLEditPropertiesDialog   GtkHTMLEditPropertiesDialog;
 
 #include <gtkhtml.h>
 #include <bonobo.h>
-#include <libgnomeui/gnome-icon-theme.h>
 #include "persist-stream.h"
 #include "htmlgdkpainter.h"
 #include "search.h"
@@ -111,7 +110,7 @@ struct _GtkHTMLControlData {
 	GtkWidget *file_dialog;
 	gboolean file_html;
 
-	GnomeIconTheme *icon_theme;
+	GtkListStore *paragraph_style_store;
 };
 
 GtkHTMLControlData * gtk_html_control_data_new       (GtkHTML *html, GtkWidget *vbox);
