@@ -236,14 +236,14 @@ html_object_real_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed
 static gint
 calc_min_width (HTMLObject *o, HTMLPainter *painter)
 {
-	html_object_calc_size (o, painter, FALSE);
+	html_object_calc_size (o, painter, NULL);
 	return o->width;
 }
 
 static gint
 calc_preferred_width (HTMLObject *o, HTMLPainter *painter)
 {
-	html_object_calc_size (o, painter, FALSE);
+	html_object_calc_size (o, painter, NULL);
 	return o->width;
 }
 	
@@ -322,9 +322,11 @@ get_bg_color (HTMLObject *o,
 	if (o->parent)
 		return html_object_get_bg_color (o->parent, p);	
 
-	if (p->widget && GTK_IS_HTML (p->widget))
-		return &((html_colorset_get_color (GTK_HTML (p->widget)->engine->settings->color_set,
-						   HTMLBgColor))->color);
+	if (p->widget && GTK_IS_HTML (p->widget)) {
+		HTMLEngine *e = html_object_engine (o, GTK_HTML (p->widget)->engine);
+		return &((html_colorset_get_color (e->settings->color_set, HTMLBgColor))->color);
+	}
+
 	return NULL;
 }
 
@@ -378,7 +380,7 @@ relayout (HTMLObject *self,
 		self->y -= prev_ascent + prev_descent;
 	}
 
-	changed = html_object_calc_size (self, engine->painter, FALSE);
+	changed = html_object_calc_size (self, engine->painter, NULL);
 
 	if (prev_width == self->width
 	    && prev_ascent == self->ascent
@@ -621,19 +623,37 @@ html_object_real_cursor_forward (HTMLObject *self, HTMLCursor *cursor)
 		return FALSE;
 }
 
+static gboolean
+html_cursor_allow_zero_offset (HTMLCursor *cursor, HTMLObject *o)
+{
+	if (cursor->offset == 1) {
+		HTMLObject *prev;
+
+		prev = html_object_prev_not_slave (o);
+		if (!prev || HTML_IS_CLUEALIGNED (prev))
+			return TRUE;
+		else {
+			while (prev && !html_object_accepts_cursor (prev))
+				prev = html_object_prev_not_slave (prev);
+
+			if (!prev)
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static gboolean
 html_object_real_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 {
-	HTMLObject *prev;
-
 	g_assert (self);
 	g_assert (cursor->object == self);
 
 	if (html_object_is_container (self))
 		return FALSE;
 
-	if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
-							  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
+	if (cursor->offset > 1 || html_cursor_allow_zero_offset (cursor, self)) {
 		cursor->offset --;
 		cursor->position --;
 		return TRUE;
@@ -653,7 +673,7 @@ html_object_real_cursor_right (HTMLObject *self, HTMLPainter *painter, HTMLCurso
 	if (html_object_is_container (self))
 		return FALSE;
 
-	if (dir == HTML_DIRECTION_LTR) {
+	if (dir != HTML_DIRECTION_RTL) {
 		gint len;
 
 		len = html_object_get_length (self);
@@ -664,10 +684,7 @@ html_object_real_cursor_right (HTMLObject *self, HTMLPainter *painter, HTMLCurso
 			return TRUE;
 		}
 	} else {
-		HTMLObject *prev;
-
-		if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
-								  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
+		if (cursor->offset > 1 || html_cursor_allow_zero_offset (cursor, self)) {
 			cursor->offset --;
 			cursor->position --;
 			return TRUE;
@@ -688,10 +705,8 @@ html_object_real_cursor_left (HTMLObject *self, HTMLPainter *painter, HTMLCursor
 	if (html_object_is_container (self))
 		return FALSE;
 
-	if (dir == HTML_DIRECTION_LTR) {
-		HTMLObject *prev;
-		if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
-								  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
+	if (dir != HTML_DIRECTION_RTL) {
+		if (cursor->offset > 1 || html_cursor_allow_zero_offset (cursor, self)) {
 			cursor->offset --;
 			cursor->position --;
 			return TRUE;
@@ -1657,9 +1672,14 @@ move_object_downtree_cursor (HTMLObject *obj, HTMLObject * (*down_fn ) (HTMLObje
 	if (!obj && last_obj) {
 		obj = last_obj;
 
-		while ((obj = (*next_fn) (obj)))
+		while ((obj = (*next_fn) (obj))) {
 			if (html_object_accepts_cursor (obj))
 				break;
+			last_obj = obj;
+			if ((obj = move_object_downtree_cursor (obj, down_fn, next_fn)))
+				break;
+			obj = last_obj;
+		}
 	}
 
 	return obj;
