@@ -2966,8 +2966,8 @@ static HTMLMagicInsertMatch mim [] = {
 	{ "(news|telnet|nntp|file|http|ftp|sftp|https)://([-a-z0-9]+(:[-a-z0-9]+)?@)?[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(([.])?/[-a-z0-9_$.+!*(),;:@%&=?/~#']*[^]'.}>\\) ,?!;:\"]?)?", NULL, NULL },
 	{ "www\\.[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(([.])?/[-A-Za-z0-9_$.+!*(),;:@%&=?/~#]*[^]'.}>\\) ,?!;:\"]?)?", NULL, "http://" },
 	{ "ftp\\.[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(([.])?/[-A-Za-z0-9_$.+!*(),;:@%&=?/~#]*[^]'.}>\\) ,?!;:\"]?)?", NULL, "ftp://" },
-	{ "h323:([-_a-z0-9.'\\+]+)?(@([-_a-z0-9.%=?]+|([0-9]{1,3}\.){3}[0-9]{1,3})?)?(:[0-9]{1,5})?", NULL, NULL },
-	{ "sip:([-_a-z0-9.'\\+]+)?(@([-_a-z0-9.%=?]+|([0-9]{1,3}\.){3}[0-9]{1,3})?)?(:[0-9]{1,5})?", NULL, NULL },
+	{ "h323:([-_a-z0-9.'\\+]+)?(@([-_a-z0-9.%=?]+|([0-9]{1,3}){3}[0-9]{1,3})?)?(:[0-9]{1,5})?", NULL, NULL },
+	{ "sip:([-_a-z0-9.'\\+]+)?(@([-_a-z0-9.%=?]+|([0-9]{1,3}){3}[0-9]{1,3})?)?(:[0-9]{1,5})?", NULL, NULL },
 	{ "[-_a-z0-9.'\\+]+@[-_a-z0-9.%=?]+", NULL, "mailto:" }
 };
 
